#include "stdafx.h"
#include "Registry.h"

////////////////////////////////////////////////////////
//
// Internal helper functions prototypes
//

// Set the given key and its value.
BOOL setKeyAndValue(CString pszPath, CString szSubkey, CString szValue) ;

// Convert a CLSID into a char string.
void CLSIDtochar(const CLSID& clsid, CString * szCLSID,	int length);

// Delete szKeyChild and all of its descendents.
LONG recursiveDeleteKey(HKEY hKeyParent, CString szKeyChild) ;

////////////////////////////////////////////////////////
//
// Constants
//

// Size of a CLSID as a string
const int CLSID_STRING_SIZE = 39 ;

/////////////////////////////////////////////////////////
//
// Public function implementation
//

//
// Register the component in the registry.
//
HRESULT RegisterServer(HMODULE hModule,            // DLL module handle
	const CLSID& clsid,         // Class ID
	const CString szFriendlyName, // Friendly Name
	const CString szVerIndProgID, // Programmatic
	const CString szProgID)       //   IDs
{
	// Get server location.
	WCHAR szModule[512] ;
	DWORD dwResult =
		::GetModuleFileName(hModule, szModule, sizeof(szModule)/sizeof(WCHAR)) ;
	assert(dwResult != 0) ;

	// Convert the CLSID into a char.
	CString szCLSID[CLSID_STRING_SIZE] ;
	CLSIDtochar(clsid, szCLSID, sizeof(szCLSID)) ;

	// Build the key CLSID\\{...}
	CString szKey;
	szKey.Append(_T("CLSID\\"));
	szKey.Append(szCLSID->GetString()) ;

	// Add the CLSID to the registry.
	setKeyAndValue(szKey.GetString(), NULL, szFriendlyName) ;

	// Add the server filename subkey under the CLSID key.
	setKeyAndValue(szKey.GetString(), _T("InprocServer32"), szModule) ;

	// Add the ProgID subkey under the CLSID key.
	setKeyAndValue(szKey.GetString(), _T("ProgID"), szProgID) ;

	// Add the version-independent ProgID subkey under CLSID key.
	setKeyAndValue(szKey.GetString(), _T("VersionIndependentProgID"),
		szVerIndProgID) ;

	// Add the version-independent ProgID subkey under HKEY_CLASSES_ROOT.
	setKeyAndValue(szVerIndProgID, NULL, szFriendlyName) ; 
	setKeyAndValue(szVerIndProgID, _T("CLSID"), szCLSID->GetString()) ;
	setKeyAndValue(szVerIndProgID, _T("CurVer"), szProgID) ;

	// Add the versioned ProgID subkey under HKEY_CLASSES_ROOT.
	setKeyAndValue(szProgID, NULL, szFriendlyName) ; 
	setKeyAndValue(szProgID, _T("CLSID"), szCLSID->GetString()) ;

	return S_OK ;
}

//
// Remove the component from the registry.
//
LONG UnregisterServer(const CLSID& clsid,         // Class ID
	CString szVerIndProgID, // Programmatic
	CString szProgID)       //   IDs
{
	// Convert the CLSID into a char.
	CString szCLSID[CLSID_STRING_SIZE] ;
	CLSIDtochar(clsid, szCLSID, sizeof(szCLSID)) ;

	// Build the key CLSID\\{...}
	CString szKey;
	szKey.Append(_T("CLSID\\"));
	szKey.Append(szCLSID->GetString());

	// Delete the CLSID Key - CLSID\{...}
	LONG lResult = recursiveDeleteKey(HKEY_CLASSES_ROOT, szKey) ;
	assert((lResult == ERROR_SUCCESS) ||
		(lResult == ERROR_FILE_NOT_FOUND)) ; // Subkey may not exist.

	// Delete the version-independent ProgID Key.
	lResult = recursiveDeleteKey(HKEY_CLASSES_ROOT, szVerIndProgID) ;
	assert((lResult == ERROR_SUCCESS) ||
		(lResult == ERROR_FILE_NOT_FOUND)) ; // Subkey may not exist.

	// Delete the ProgID key.
	lResult = recursiveDeleteKey(HKEY_CLASSES_ROOT, szProgID) ;
	assert((lResult == ERROR_SUCCESS) ||
		(lResult == ERROR_FILE_NOT_FOUND)) ; // Subkey may not exist.

	return S_OK ;
}

///////////////////////////////////////////////////////////
//
// Internal helper functions
//

// Convert a CLSID to a char string.
void CLSIDtochar(const CLSID& clsid, CString * szCLSID,	int length)
{
	assert(length >= CLSID_STRING_SIZE) ;
	// Get CLSID
	LPOLESTR wszCLSID = NULL ;
	HRESULT hr = StringFromCLSID(clsid, &wszCLSID) ;
	assert(SUCCEEDED(hr)) ;

	// Covert from wide characters to non-wide.
	szCLSID->SetString(OLE2W(wszCLSID));

	// Free memory.
	CoTaskMemFree(wszCLSID) ;
}

//
// Delete a key and all of its descendents.
//
LONG recursiveDeleteKey(HKEY hKeyParent,           // Parent of key to delete
	CString lpszKeyChild)  // Key to delete
{
	// Open the child.
	HKEY hKeyChild ;
	LONG lRes = RegOpenKeyEx(hKeyParent, lpszKeyChild, 0,
		KEY_ALL_ACCESS, &hKeyChild) ;
	if (lRes != ERROR_SUCCESS)
	{
		return lRes ;
	}

	// Enumerate all of the decendents of this child.
	FILETIME time ;
	CString szBuffer[256] ;
	DWORD dwSize = 256 ;
	while (RegEnumKeyEx(hKeyChild, 0, (LPWSTR)szBuffer, &dwSize, NULL,
		NULL, NULL, &time) == S_OK)
	{
		// Delete the decendents of this child.
		lRes = recursiveDeleteKey(hKeyChild, szBuffer->GetString()) ;
		if (lRes != ERROR_SUCCESS)
		{
			// Cleanup before exiting.
			RegCloseKey(hKeyChild) ;
			return lRes;
		}
		dwSize = 256 ;
	}

	// Close the child.
	RegCloseKey(hKeyChild) ;

	// Delete this child.
	return RegDeleteKey(hKeyParent, lpszKeyChild) ;
}

//
// Create a key and set its value.
//   - This helper function was borrowed and modifed from
//     Kraig Brockschmidt's book Inside OLE.
//
BOOL setKeyAndValue(CString szKey, CString szSubkey,	CString szValue)
{
	HKEY hKey;
	CString szKeyBuf;
	szKeyBuf.SetString(szKey);

	// Copy keyname into buffer.

	// Add subkey name to buffer.
	if (!szSubkey.IsEmpty())
	{
		szKeyBuf.Append(_T("\\"));
		szKeyBuf.Append(szSubkey) ;
	}

	// Create and open key and subkey.
	long lResult = RegCreateKeyEx(HKEY_CLASSES_ROOT ,
		szKeyBuf, 
		0, NULL, REG_OPTION_NON_VOLATILE,
		KEY_ALL_ACCESS, NULL, 
		&hKey, NULL) ;
	if (lResult != ERROR_SUCCESS)
	{
		return FALSE ;
	}

	// Set the Value.
	if (!szValue.IsEmpty())
	{
		LPWSTR _s = (LPWSTR)szValue.GetString();
		int Length = wcslen(_s) * sizeof(WCHAR);
		RegSetValueEx(hKey, NULL, 0, REG_SZ, (BYTE *) _s, Length +1) ;
	}

	RegCloseKey(hKey) ;
	return TRUE ;
}
